@@ -23,7 +23,7 @@ WWW::YoutubeViewer - A very easy interface to YouTube.
 
 =cut
 
-our $VERSION = '3.5.8';
+our $VERSION = '3.5.9';
 
 =head1 SYNOPSIS
 
@@ -72,7 +72,7 @@ my %valid_options = (
     subscriptions_order => {valid => [qw(alphabetical relevance unread)], default => undef},
 
     # Others
-    debug       => {valid => [0 .. 2],     default => 0},
+    debug       => {valid => [0 .. 3],     default => 0},
     lwp_timeout => {valid => [qr/^\d+\z/], default => 1},
     key         => {valid => [qr/^.{5}/],  default => undef},
     config_dir  => {valid => [qr/^./],     default => q{.}},
@@ -570,7 +570,8 @@ sub _fallback_extract_urls {
     }
 
     if ($self->get_debug) {
-        say STDERR ":: Found ", scalar(@formats), " streaming URLs.";
+        my $count = scalar(@formats);
+        say STDERR ":: Found $count streaming URLs...";
     }
 
     return @formats;
@@ -641,9 +642,19 @@ sub _group_keys_with_values {
 sub _old_extract_streaming_urls {
     my ($self, $info, $videoID) = @_;
 
+    if ($self->get_debug) {
+        say STDERR ":: Using `url_encoded_fmt_stream_map` to extract the streaming URLs...";
+    }
+
     my %stream_map = $self->parse_query_string($info->{url_encoded_fmt_stream_map}, multi => 1);
     my %adaptive_fmts = $self->parse_query_string($info->{adaptive_fmts}, multi => 1);
 
+    if ($self->get_debug >= 2) {
+        require Data::Dump;
+        Data::Dump::pp(\%stream_map);
+        Data::Dump::pp(\%adaptive_fmts);
+    }
+
     my @results;
 
     push @results, $self->_group_keys_with_values(%stream_map);
@@ -652,6 +663,10 @@ sub _old_extract_streaming_urls {
     foreach my $video (@results) {
         if (exists $video->{s}) {    # has an encrypted signature :(
 
+            if ($self->get_debug) {
+                say STDERR ":: Detected an encrypted signature...";
+            }
+
             my @formats = $self->_fallback_extract_urls($videoID);
 
             foreach my $format (@formats) {
@@ -668,6 +683,11 @@ sub _old_extract_streaming_urls {
     }
 
     if ($info->{livestream} or $info->{live_playback}) {
+
+        if ($self->get_debug) {
+            say STDERR ":: Live stream detected...";
+        }
+
         if (my @formats = $self->_fallback_extract_urls($videoID)) {
             @results = @formats;
         }
@@ -681,6 +701,11 @@ sub _old_extract_streaming_urls {
         }
     }
 
+    if ($self->get_debug) {
+        my $count = scalar(@results);
+        say STDERR ":: Found $count streaming URLs...";
+    }
+
     return @results;
 }
 
@@ -692,7 +717,7 @@ sub _extract_streaming_urls {
     }
 
     if ($self->get_debug) {
-        say STDERR ":: Using the newer format to extract the streaming URLs...";
+        say STDERR ":: Using `player_response` to extract the streaming URLs...";
     }
 
     my $json = $self->parse_json_string($info->{player_response});
@@ -704,19 +729,23 @@ sub _extract_streaming_urls {
 
     my @results;
     if (exists $json->{streamingData}) {
-        if (exists $json->{streamingData}{adaptiveFormats}) {
-            push @results, @{$json->{streamingData}{adaptiveFormats}};
+
+        my $streamingData = $json->{streamingData};
+
+        if (exists $streamingData->{adaptiveFormats}) {
+            push @results, @{$streamingData->{adaptiveFormats}};
         }
 
-        if (exists $json->{streamingData}{formats}) {
-            push @results, @{$json->{streamingData}{formats}};
+        if (exists $streamingData->{formats}) {
+            push @results, @{$streamingData->{formats}};
         }
     }
 
     foreach my $item (@results) {
 
         if (exists $item->{cipher} and not exists $item->{url}) {
-            my (%data) = $self->parse_query_string($item->{cipher});
+
+            my %data = $self->parse_query_string($item->{cipher});
 
             $item->{url} = $data{url} if defined($data{url});
 
@@ -739,6 +768,25 @@ sub _extract_streaming_urls {
     # Keep only streams with contentLength > 0.
     @results = grep { exists($_->{contentLength}) and $_->{contentLength} > 0 } @results;
 
+    # Detect livestream
+    if (!@results and exists($json->{streamingData}) and exists($json->{streamingData}{hlsManifestUrl})) {
+
+        if ($self->get_debug) {
+            say STDERR ":: Live stream detected...";
+        }
+
+        @results = $self->_fallback_extract_urls($videoID);
+
+        if (!@results) {
+            push @results,
+              {
+                itag => 38,
+                type => "video/ts",
+                url  => $json->{streamingData}{hlsManifestUrl},
+              };
+        }
+    }
+
     if ($self->get_debug) {
         my $count = scalar(@results);
         say STDERR ":: Found $count streaming URLs...";
@@ -775,13 +823,6 @@ sub get_streaming_urls {
         }
     }
 
-    if ($self->get_debug == 3) {
-        require Data::Dump;
-        Data::Dump::pp(\%info);
-        Data::Dump::pp(\@streaming_urls);
-        Data::Dump::pp(\@caption_urls);
-    }
-
     # Try again with youtube-dl
     if (!@streaming_urls or $info{status} =~ /fail|error/i) {
         @streaming_urls = $self->_fallback_extract_urls($videoID);
@@ -815,13 +856,19 @@ sub get_streaming_urls {
     if (!@streaming_urls) {
         push @streaming_urls,
           {
-            itag    => 22,
-            quality => "hd720",
-            type    => "video/mp4",
-            url     => "https://www.youtube.com/watch?v=$videoID",
+            itag => 38,
+            type => "video/mp4",
+            url  => "https://www.youtube.com/watch?v=$videoID",
           };
     }
 
+    if ($self->get_debug >= 2) {
+        require Data::Dump;
+        Data::Dump::pp(\%info) if ($self->get_debug >= 3);
+        Data::Dump::pp(\@streaming_urls);
+        Data::Dump::pp(\@caption_urls);
+    }
+
     return (\@streaming_urls, \@caption_urls, \%info);
 }
 
